from __future__ import annotations

import logging
import pathlib
from typing import TYPE_CHECKING

from vestapol.destinations import base_destination

if TYPE_CHECKING:
    from vestapol.web_resources.base_resource import BaseResource

logger = logging.getLogger(__name__)


class Local(base_destination.BaseDestination):
    def write_data(self, data: str, pathname: str):
        abs_pathname = self.create_local_folders(pathname)
        logger.debug(f"Writing data to {abs_pathname}")
        with abs_pathname.open(mode="w") as file:
            file.write(data)

    @staticmethod
    def create_local_folders(pathname: str):
        root_dir = pathlib.Path.cwd().resolve()
        extract_dir = root_dir / "data"
        abs_pathname = extract_dir / pathname
        abs_pathname.parent.mkdir(parents=True, exist_ok=True)
        return abs_pathname

    def create_table(self, resource: BaseResource):
        return super().create_table(resource)



from vestapol.web_resources import base_resource


class GitHubResource(base_resource.BaseResource):
    base_url = "https://raw.githubusercontent.com/"

    def __init__(
        self,
        name: str,
        endpoint: str,
        version: str,
        response_format_tag: str,
        external_data_format_tag: str,
        response_filename: str,
    ):
        super().__init__(
            name,
            self.base_url,
            endpoint,
            version,
            response_format_tag,
            external_data_format_tag,
            response_filename,
        )



from __future__ import annotations

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import TYPE_CHECKING

from vestapol.web_resources import base_resource
from vestapol.writers import text_writer

if TYPE_CHECKING:
    from vestapol.destinations.base_destination import BaseDestination


class CSVResource(base_resource.BaseResource):
    response_format_tag = external_data_format_tag = "csv"
    response_filename = "data.csv"

    def __init__(
        self,
        name: str,
        base_url: Optional[str],
        endpoint: Optional[str],
        version: str,
        skip_leading_rows: int = 0,
        query_params: dict = None,
        request_headers: dict = None,
        manual_schema: List[Dict] = None,
    ):
        self.name = name
        self.base_url = base_url
        self.endpoint = endpoint
        self.version = version
        self.skip_leading_rows = skip_leading_rows
        self.query_params = query_params
        self.request_headers = request_headers
        self.manual_schema = manual_schema

        super().__init__(
            self.name,
            self.base_url,
            self.endpoint,
            self.version,
            self.response_format_tag,
            self.external_data_format_tag,
            self.response_filename,
            self.query_params,
            self.request_headers,
            self.manual_schema,
        )

    def write_data(self, data, destination: BaseDestination):
        self.write_string(data, destination, data_path=None)

    def write_string(
        self,
        data: str,
        destination: BaseDestination,
        data_path: List[Tuple[str, Any]] = None,
    ):
        pathname = self.get_pathname(data_path)
        text_writer.write_text(data, pathname, destination)



import logging

logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s %(name)s %(levelname)s:%(message)s"
)

try:
    from dotenv import load_dotenv

    load_dotenv()
except ImportError:
    pass



import logging

import requests

logger = logging.getLogger(__name__)

JSON_FORMAT_TAG = "json"
CSV_FORMAT_TAG = "csv"


def get_api_data(
    url: str, response_format_tag: str, query_params: dict, request_headers: dict
):
    logger.debug(f"GET {url}")

    try:
        response = requests.get(url, params=query_params, headers=request_headers)
        logger.debug(response.url)
        response.raise_for_status()
    except Exception as e:
        logger.error(e)
        raise

    if response_format_tag == JSON_FORMAT_TAG:
        return response.json()

    elif response_format_tag == CSV_FORMAT_TAG:
        return response.text

    else:
        logger.error(f"Unsupported format tag: {response_format_tag}")
        raise ValueError(f"Unsupported format tag: {response_format_tag}")



from __future__ import annotations

import logging
import os
import pathlib
from typing import TYPE_CHECKING

from google.cloud import bigquery
from google.cloud import storage

from vestapol import external_tables
from vestapol.destinations import base_destination


if TYPE_CHECKING:
    from vestapol.web_resources.base_resource import BaseResource

logger = logging.getLogger(__name__)


class GoogleCloudPlatform(base_destination.BaseDestination):
    def __init__(
        self,
        gcs_bucket_name: str = None,
        gcs_root_prefix: str = None,
        gbq_project_id: str = None,
        gbq_dataset_id: str = None,
        gbq_dataset_location: str = None,
    ):
        self.gcs_bucket_name = gcs_bucket_name or os.environ["GCS_BUCKET_NAME"]
        self.gcs_root_prefix = gcs_root_prefix or os.environ["GCS_ROOT_PREFIX"]
        self.gcs_root_prefix_fq = f"gs://{self.gcs_bucket_name}/{self.gcs_root_prefix}/"
        self.gbq_project_id = gbq_project_id or os.environ["GBQ_PROJECT_ID"]
        self.gbq_dataset_id = gbq_dataset_id or os.environ["GBQ_DATASET_ID"]
        self.gbq_dataset_location = (
            gbq_dataset_location or os.environ["GBQ_DATASET_LOCATION"]
        )

    def write_data(self, data: str, pathname: str):
        destination_blob_name = f"{self.gcs_root_prefix}/{str(pathname)}"
        logger.debug(
            f"Uploading data to gs://{self.gcs_bucket_name}/{destination_blob_name}"
        )
        storage_client = storage.Client()
        bucket = storage_client.bucket(self.gcs_bucket_name)
        blob = bucket.blob(destination_blob_name)
        blob.upload_from_string(data, timeout=120)

    def create_table(self, resource: BaseResource):
        common_prefix = pathlib.Path(
            resource.name, resource.external_data_format_tag, resource.version
        )

        sources = [f"{common_prefix}/*/{resource.response_filename}"]

        source_uris = [f"{self.gcs_root_prefix_fq}{source}" for source in sources]

        source_uri_prefix_fq = f"{self.gcs_root_prefix_fq}{common_prefix}/"

        tablename = resource.name

        if resource.manual_schema is not None:
            table_schema = []
            for s in resource.manual_schema:
                if "fields" in s.keys():
                    s["fields"] = [bigquery.SchemaField(**f) for f in s["fields"]]
                table_schema.append(bigquery.SchemaField(**s))
        else:
            table_schema = None

        external_tables.create_gcp_table(
            resource.external_data_format_tag,
            self.gbq_project_id,
            self.gbq_dataset_id,
            self.gbq_dataset_location,
            tablename,
            source_uri_prefix_fq,
            source_uris,
            table_schema,
            resource.skip_leading_rows,
        )

        tablename_fq = f"{self.gbq_project_id}.{self.gbq_dataset_id}.{tablename}"

        return tablename_fq



from __future__ import annotations

from abc import ABC
from abc import abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from vestapol.web_resources.base_resource import BaseResource


class BaseDestination(ABC):
    @abstractmethod
    def write_data(self, data: str, pathname: str):
        pass

    @abstractmethod
    def create_table(self, resource: BaseResource):
        """
        Implementations of method should return the fully-qualified table name
        of the created table
        :rtype: str
        """
        raise NotImplementedError



from __future__ import annotations

from abc import abstractmethod
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import TYPE_CHECKING

from vestapol.web_resources import base_resource
from vestapol.writers import json_writer

if TYPE_CHECKING:
    from vestapol.destinations.base_destination import BaseDestination


class JSONResource(base_resource.BaseResource):
    response_format_tag = "json"
    external_data_format_tag = "jsonl"
    response_filename = "data.json"

    def __init__(
        self,
        name,
        base_url,
        endpoint,
        version,
        query_params=None,
        request_headers=None,
        manual_schema=None,
    ):
        self.name: str = name
        self.base_url: str = base_url
        self.endpoint: str = endpoint
        self.version: str = version
        self.query_params: Optional[dict] = query_params
        self.request_headers: Optional[dict] = request_headers
        self.manual_schema: Optional[List[Dict]] = manual_schema
        super().__init__(
            self.name,
            self.base_url,
            self.endpoint,
            self.version,
            self.response_format_tag,
            self.external_data_format_tag,
            self.response_filename,
            self.query_params,
            self.request_headers,
            self.manual_schema,
        )

    def load(self, destination: BaseDestination):
        data = self.request_data()
        self.write_data(data, destination)
        self.unnest_data(data, destination)
        return data

    def write_data(self, api_data, destination: BaseDestination):
        json_writer.write_json(
            api_data, self.response_target_prefix / self.response_filename, destination
        )

    @abstractmethod
    def unnest_data(self, data, destination):
        pass

    def write_list(
        self,
        data: List[Dict],
        destination: BaseDestination,
        data_path: List[Tuple[str, Any]] = None,
    ):
        if not isinstance(data, list):
            raise TypeError(f"Expected a list, received {type(data)}")

        pathname = self.get_pathname(data_path)
        json_writer.write_jsonl(data=data, pathname=pathname, destination=destination)

    def write_dict(
        self,
        data: Dict,
        destination: BaseDestination,
        data_path: List[Tuple[str, Any]] = None,
    ):
        if not isinstance(data, dict):
            raise TypeError(f"Expected a dict, received {type(data)}")

        pathname = self.get_pathname(data_path)
        json_writer.write_json(data=data, pathname=pathname, destination=destination)



from __future__ import annotations

import pathlib
from abc import ABC
from abc import abstractmethod
from typing import Any
from typing import List
from typing import Optional
from typing import Tuple
from typing import TYPE_CHECKING

from pendulum import DateTime

from vestapol.api import api

if TYPE_CHECKING:
    from vestapol.destinations.base_destination import BaseDestination


class BaseResource(ABC):
    def __init__(
        self,
        name: str,
        base_url: Optional[str],
        endpoint: Optional[str],
        version: str,
        response_format_tag: str,
        external_data_format_tag: str,
        response_filename: str = "data",
        query_params: dict = None,
        request_headers: dict = None,
        manual_schema=None,
    ):
        self.name = name
        self.base_url = base_url
        self.endpoint = endpoint
        self.version = version
        self.response_format_tag = response_format_tag
        self.external_data_format_tag = external_data_format_tag
        self.response_filename = response_filename
        self.query_params = query_params
        self.request_headers = request_headers
        self.manual_schema = manual_schema
        self.requested_at = DateTime.utcnow().replace(microsecond=0)
        self.skip_leading_rows: int = 0

    def load(self, destination: BaseDestination):
        data = self.request_data()
        self.write_data(data, destination)
        return data

    def request_data(self):
        return api.get_api_data(
            f"{self.base_url}{self.endpoint}",
            self.response_format_tag,
            self.query_params,
            self.request_headers,
        )

    @abstractmethod
    def write_data(self, data, destination: BaseDestination):
        pass

    def get_response_root(self, format_tag: str):
        return pathlib.Path(
            self.name, format_tag, self.version, self.requested_at_hive_path
        )

    @property
    def response_target_prefix(self):
        self._response_target_prefix = self.get_response_root(self.response_format_tag)
        return self._response_target_prefix

    @property
    def requested_at_hive_path(self):
        return f'requested_at={self.requested_at.strftime("%Y-%m-%d %H:%M:%S")}'

    @staticmethod
    def get_hive_path(data_path: List[Tuple[str, Any]]) -> str:

        prefix_components = []
        for x in data_path:
            prefix_components.append(f"{x[0]}={x[1]}")

        hive_path = "/".join(prefix_components)

        return hive_path

    def get_pathname(self, data_path: List[Tuple[str, Any]] = None) -> str:
        target_prefix = self.get_response_root(self.external_data_format_tag)

        if data_path:
            hive_path = self.get_hive_path(data_path)
            pathname = target_prefix / hive_path / self.response_filename
        else:
            pathname = target_prefix / self.response_filename

        return pathname



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vestapol',
 'vestapol.api',
 'vestapol.destinations',
 'vestapol.web_resources',
 'vestapol.writers']

package_data = \
{'': ['*']}

install_requires = \
['google-cloud-bigquery>=2.31,<3.0',
 'google-cloud-storage>=1.43,<2.0',
 'pendulum>=2.1,<3.0',
 'requests>=2.26,<3.0']

setup_kwargs = {
    'name': 'vestapol',
    'version': '0.0.16',
    'description': 'Python package that loads data from the web and deploys a corresponding external table definition, so that the data can be queried using standard SQL.',
    'long_description': '# vestapol\n\nvestapol is a Python package that loads data from the web and deploys a corresponding external table definition, so that the data can be queried using standard SQL.\n\n["Vestapol"](https://www.youtube.com/watch?v=SKQG-JGyn7U) is an open D Major tuning for the guitar. It is named after a 19th-century composition distributed in some of the earliest instructional guides for guitar.\n\n## Usage\n\n```python\nfrom vestapol.web_resources.csv_resource import CSVResource\nfrom vestapol.destinations.gcp_destination import GoogleCloudPlatform\n\n\nnyt_covid_data_2022 = CSVResource(\n    name="nyt_covid19_us_counties_2022",\n    base_url="https://raw.githubusercontent.com/",\n    endpoint="nytimes/covid-19-data/master/rolling-averages/us-counties-2022.csv",\n    version="v0",\n    skip_leading_rows=1,\n)\n\ndestination = GoogleCloudPlatform()\n\nnyt_covid_data_2022.load(destination)\ntablename = destination.create_table(nyt_covid_data_2022)\n\n\nfrom google.cloud import bigquery\n\nclient = bigquery.Client()\nquery = f"""\n    select date, state, county, cases_avg_per_100k\n    from `{tablename}`\n    where requested_at = \'{nyt_covid_data_2022.requested_at}\'\n    limit 5\n"""\nquery_job = client.query(query)\nfor row in query_job.result():\n    print(row)\n```\n\n\n## Prerequisites\n\nInstallation of this project requires [Poetry](https://python-poetry.org/docs/) and Python version 3.8+.\n\n\n## Installation\n\nInstall vestapol and its dependencies by running:\n\n```shell\npoetry install\n```\n\n## Testing\n\nRun tests with the following command:\n\n```shell\npoetry run pytest\n```\n\n## Environment Variables\n\n- `GCS_BUCKET_NAME`: the Google Cloud Storage bucket where data is loaded (e.g. `inq-warehouse-waligob`)\n- `GCS_ROOT_PREFIX`: the GCS prefix where data is loaded (e.g. `data_catalog`)\n- `GBQ_PROJECT_ID`: the BigQuery project identifier (e.g. `inq-warehouse`)\n- `GBQ_DATASET_ID`: the BigQuery dataset where external tables will be created (e.g. `data_catalog_waligob`)\n- `GBQ_DATASET_LOCATION`: the BigQuery dataset location (e.g. `US`)\n- `GOOGLE_APPLICATION_CREDENTIALS=`: location of the GCS service account keyfile (e.g. `~/inq-warehouse-f0962a57089e-inf.json`)\n\n\n## Publishing to PyPI\n\nInstructions for pushing new versions of `vestapol` to PyPI:\n\n1. Update `CHANGELOG.md`. Include Additions, Fixes, and Changes.\n\n2. Update project version using either a valid [PEP 440 string](https://peps.python.org/pep-0440/) or a [valid bump rule](https://python-poetry.org/docs/master/cli/#version) following [Semantic Versioning](http://semver.org/).\n\n```shell\n    poetry version <version string or bump rule>\n```\n\n3. Create a [release](https://docs.github.com/en/repositories/releasing-projects-on-github/managing-releases-in-a-repository#creating-a-release) and check the CD Pipeline [action](https://github.com/phillymedia/vestapol/actions/workflows/release.yml) to ensure that the project was built and published to PyPI successfully.\n',
    'author': 'Brian Waligorski',
    'author_email': 'bwaligorski@inquirer.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/phillymedia/vestapol',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.12,<3.11',
}


setup(**setup_kwargs)



import json
import logging
from typing import Dict
from typing import List

from vestapol.destinations.base_destination import BaseDestination

logger = logging.getLogger(__name__)


def write_json(data: dict, pathname: str, destination: BaseDestination):
    data_string = json.dumps(data)
    destination.write_data(data_string, pathname)


def write_jsonl(data: List[Dict], pathname: str, destination: BaseDestination):
    data_string = "\n".join([json.dumps(record) for record in data])
    destination.write_data(data_string, pathname)



from typing import List
from typing import Optional

from google.cloud import bigquery


def get_dataset(
    client: bigquery.Client, project_id: str, dataset_id: str, dataset_location: str
):
    dataset = bigquery.Dataset(f"{project_id}.{dataset_id}")
    dataset.location = dataset_location
    dataset = client.create_dataset(dataset, exists_ok=True, timeout=30)
    return dataset


def get_external_data_configuration(
    source_uri_prefix_fq: str,
    source_uris: List[str],
    source_format: str,
    table_schema: Optional[List[bigquery.SchemaField]] = None,
    skip_leading_rows: int = 0,
):

    bq_source_format = {"jsonl": "NEWLINE_DELIMITED_JSON", "csv": "CSV"}[source_format]

    external_config = bigquery.ExternalConfig(bq_source_format)

    # autodetect schema unless a manually defined one is provided
    if table_schema is not None:
        external_config.schema = table_schema
    else:
        external_config.autodetect = True

    hive_partitioning_opts = bigquery.external_config.HivePartitioningOptions()
    hive_partitioning_opts.mode = "AUTO"
    hive_partitioning_opts.source_uri_prefix = source_uri_prefix_fq

    external_config.hive_partitioning = hive_partitioning_opts

    external_config.source_uris = source_uris

    if bq_source_format == "CSV":
        external_config.csv_options.skip_leading_rows = skip_leading_rows

    return external_config


def create_gcp_table(
    source_format: str,
    project_id: str,
    dataset_id: str,
    dataset_location: str,
    table_id: str,
    source_uri_prefix_fq: str,
    source_uris: List[str],
    table_schema: Optional[List[bigquery.SchemaField]],
    skip_leading_rows: int = 0,
):

    client = bigquery.Client()
    dataset = get_dataset(client, project_id, dataset_id, dataset_location)

    # TODO: optionally create schema from header metadata

    table = bigquery.Table(dataset.table(table_id))

    table.external_data_configuration = get_external_data_configuration(
        source_uri_prefix_fq,
        source_uris,
        source_format,
        table_schema,
        skip_leading_rows,
    )

    # Create a permanent table linked to the GCS file
    table = client.create_table(table, exists_ok=True)  # API request
